@@ -28,7 +28,7 @@ class ArchiveDownloadTask : public Task {
     ArchiveDownloadTask(QUrl url, QString final_path, QString checksumType = "", QString checksumHash = "");
     virtual ~ArchiveDownloadTask() = default;
 
-    [[nodiscard]] bool canAbort() const override { return true; }
+    bool canAbort() const override { return true; }
     void executeTask() override;
     virtual bool abort() override;
 
@@ -42,4 +42,4 @@ class ArchiveDownloadTask : public Task {
     QString m_checksum_hash;
     Task::Ptr m_task;
 };
-}  // namespace Java
\ No newline at end of file
+}  // namespace Java
