@@ -29,7 +29,7 @@ class ManifestDownloadTask : public Task {
     ManifestDownloadTask(QUrl url, QString final_path, QString checksumType = "", QString checksumHash = "");
     virtual ~ManifestDownloadTask() = default;
 
-    [[nodiscard]] bool canAbort() const override { return true; }
+    bool canAbort() const override { return true; }
     void executeTask() override;
     virtual bool abort() override;
 
@@ -43,4 +43,4 @@ class ManifestDownloadTask : public Task {
     QString m_checksum_hash;
     Task::Ptr m_task;
 };
-}  // namespace Java
\ No newline at end of file
+}  // namespace Java
