@@ -521,7 +521,7 @@ void ComponentUpdateTask::resolveDependencies(bool checkOnly)
         // change a version of something that exists
         for (auto& change : toChange) {
             // FIXME: this should not work directly with the component list
-            qCDebug(instanceProfileResolveC) << "Setting version of " << change.uid << "to" << change.equalsVersion;
+            qCDebug(instanceProfileResolveC) << "Setting version of" << change.uid << "to" << change.equalsVersion;
             auto component = componentIndex[change.uid];
             component->setVersion(change.equalsVersion);
         }
@@ -570,7 +570,7 @@ void ComponentUpdateTask::performUpdateActions()
                               component->setVersion(cv.targetVersion);
                               component->waitLoadMeta();
                           },
-                          [&component, &instance](const UpdateActionLatestRecommendedCompatible lrc) {
+                          [&component, &instance](const UpdateActionLatestRecommendedCompatible& lrc) {
                               qCDebug(instanceProfileResolveC)
                                   << instance->name() << "|"
                                   << "UpdateActionLatestRecommendedCompatible" << component->getID() << ":" << component->getVersion()
@@ -744,11 +744,10 @@ void ComponentUpdateTask::remoteLoadFailed(size_t taskIndex, const QString& msg)
         qCWarning(instanceProfileResolveC) << "Got multiple results from remote load task" << taskIndex;
         return;
     }
-    qCDebug(instanceProfileResolveC) << "Remote task" << taskIndex << "failed: " << msg;
+    qCDebug(instanceProfileResolveC) << "Remote task" << taskIndex << "failed:" << msg;
     d->remoteLoadSuccessful = false;
     taskSlot.succeeded = false;
     taskSlot.finished = true;
-    taskSlot.error = msg;
     d->remoteTasksInProgress--;
     checkIfAllFinished();
 }
@@ -769,11 +768,14 @@ void ComponentUpdateTask::checkIfAllFinished()
         QStringList allErrorsList;
         for (auto& item : d->remoteLoadStatusList) {
             if (!item.succeeded) {
-                allErrorsList.append(item.error);
+                const ComponentPtr component = d->m_profile->getComponent(item.PackProfileIndex);
+                allErrorsList.append(tr("Could not download metadata for %1 %2. Please change the version or try again later.")
+                                         .arg(component->getName(), component->m_version));
             }
         }
+        d->remoteLoadStatusList.clear();
+
         auto allErrors = allErrorsList.join("\n");
         emitFailed(tr("Component metadata update task failed while downloading from remote server:\n%1").arg(allErrors));
-        d->remoteLoadStatusList.clear();
     }
 }
