@@ -2,15 +2,16 @@
 #include "MinecraftInstance.h"
 #include "PackProfile.h"
 
-MinecraftLoadAndCheck::MinecraftLoadAndCheck(MinecraftInstance* inst, Net::Mode netmode, QObject* parent)
-    : Task(parent), m_inst(inst), m_netmode(netmode)
-{}
+MinecraftLoadAndCheck::MinecraftLoadAndCheck(MinecraftInstance* inst, Net::Mode netmode) : m_inst(inst), m_netmode(netmode) {}
 
 void MinecraftLoadAndCheck::executeTask()
 {
     // add offline metadata load task
     auto components = m_inst->getPackProfile();
-    components->reload(m_netmode);
+    if (auto result = components->reload(m_netmode); !result) {
+        emitFailed(result.error);
+        return;
+    }
     m_task = components->getCurrentTask();
 
     if (!m_task) {
@@ -42,4 +43,4 @@ bool MinecraftLoadAndCheck::abort()
         return status;
     }
     return Task::abort();
-}
\ No newline at end of file
+}
