@@ -260,6 +260,30 @@ int AccountList::count() const
     return m_accounts.count();
 }
 
+QString getAccountStatus(AccountState status)
+{
+    switch (status) {
+        case AccountState::Unchecked:
+            return QObject::tr("Unchecked", "Account status");
+        case AccountState::Offline:
+            return QObject::tr("Offline", "Account status");
+        case AccountState::Online:
+            return QObject::tr("Ready", "Account status");
+        case AccountState::Working:
+            return QObject::tr("Working", "Account status");
+        case AccountState::Errored:
+            return QObject::tr("Errored", "Account status");
+        case AccountState::Expired:
+            return QObject::tr("Expired", "Account status");
+        case AccountState::Disabled:
+            return QObject::tr("Disabled", "Account status");
+        case AccountState::Gone:
+            return QObject::tr("Gone", "Account status");
+        default:
+            return QObject::tr("Unknown", "Account status");
+    }
+}
+
 QVariant AccountList::data(const QModelIndex& index, int role) const
 {
     if (!index.isValid())
@@ -273,13 +297,10 @@ QVariant AccountList::data(const QModelIndex& index, int role) const
     switch (role) {
         case Qt::DisplayRole:
             switch (index.column()) {
-                case ProfileNameColumn: {
+                case ProfileNameColumn:
                     return account->profileName();
-                }
-
                 case NameColumn:
                     return account->accountDisplayString();
-
                 case TypeColumn: {
                     switch (account->accountType()) {
                         case AccountType::MSA: {
@@ -291,39 +312,8 @@ QVariant AccountList::data(const QModelIndex& index, int role) const
                     }
                     return tr("Unknown", "Account type");
                 }
-
-                case StatusColumn: {
-                    switch (account->accountState()) {
-                        case AccountState::Unchecked: {
-                            return tr("Unchecked", "Account status");
-                        }
-                        case AccountState::Offline: {
-                            return tr("Offline", "Account status");
-                        }
-                        case AccountState::Online: {
-                            return tr("Ready", "Account status");
-                        }
-                        case AccountState::Working: {
-                            return tr("Working", "Account status");
-                        }
-                        case AccountState::Errored: {
-                            return tr("Errored", "Account status");
-                        }
-                        case AccountState::Expired: {
-                            return tr("Expired", "Account status");
-                        }
-                        case AccountState::Disabled: {
-                            return tr("Disabled", "Account status");
-                        }
-                        case AccountState::Gone: {
-                            return tr("Gone", "Account status");
-                        }
-                        default: {
-                            return tr("Unknown", "Account status");
-                        }
-                    }
-                }
-
+                case StatusColumn:
+                    return getAccountStatus(account->accountState());
                 default:
                     return QVariant();
             }
@@ -335,11 +325,9 @@ QVariant AccountList::data(const QModelIndex& index, int role) const
             return QVariant::fromValue(account);
 
         case Qt::CheckStateRole:
-            if (index.column() == ProfileNameColumn) {
+            if (index.column() == ProfileNameColumn)
                 return account == m_defaultAccount ? Qt::Checked : Qt::Unchecked;
-            } else {
-                return QVariant();
-            }
+            return QVariant();
 
         default:
             return QVariant();
@@ -461,18 +449,14 @@ bool AccountList::loadList()
 
     // Make sure the format version matches.
     auto listVersion = root.value("formatVersion").toVariant().toInt();
-    switch (listVersion) {
-        case AccountListVersion::MojangMSA: {
-            return loadV3(root);
-        } break;
-        default: {
-            QString newName = "accounts-old.json";
-            qWarning() << "Unknown format version when loading account list. Existing one will be renamed to" << newName;
-            // Attempt to rename the old version.
-            file.rename(newName);
-            return false;
-        }
-    }
+    if (listVersion == AccountListVersion::MojangMSA)
+        return loadV3(root);
+
+    QString newName = "accounts-old.json";
+    qWarning() << "Unknown format version when loading account list. Existing one will be renamed to" << newName;
+    // Attempt to rename the old version.
+    file.rename(newName);
+    return false;
 }
 
 bool AccountList::loadV3(QJsonObject& root)
@@ -590,7 +574,7 @@ void AccountList::fillQueue()
     if (m_defaultAccount && m_defaultAccount->shouldRefresh()) {
         auto idToRefresh = m_defaultAccount->internalId();
         m_refreshQueue.push_back(idToRefresh);
-        qDebug() << "AccountList: Queued default account with internal ID " << idToRefresh << " to refresh first";
+        qDebug() << "AccountList: Queued default account with internal ID" << idToRefresh << "to refresh first";
     }
 
     for (int i = 0; i < count(); i++) {
@@ -614,7 +598,7 @@ void AccountList::requestRefresh(QString accountId)
         m_refreshQueue.removeAt(index);
     }
     m_refreshQueue.push_front(accountId);
-    qDebug() << "AccountList: Pushed account with internal ID " << accountId << " to the front of the queue";
+    qDebug() << "AccountList: Pushed account with internal ID" << accountId << "to the front of the queue";
     if (!isActive()) {
         tryNext();
     }
@@ -626,7 +610,7 @@ void AccountList::queueRefresh(QString accountId)
         return;
     }
     m_refreshQueue.push_back(accountId);
-    qDebug() << "AccountList: Queued account with internal ID " << accountId << " to refresh";
+    qDebug() << "AccountList: Queued account with internal ID" << accountId << "to refresh";
 }
 
 void AccountList::tryNext()
@@ -642,13 +626,13 @@ void AccountList::tryNext()
                     connect(m_currentTask.get(), &Task::succeeded, this, &AccountList::authSucceeded);
                     connect(m_currentTask.get(), &Task::failed, this, &AccountList::authFailed);
                     m_currentTask->start();
-                    qDebug() << "RefreshSchedule: Processing account " << account->accountDisplayString() << " with internal ID "
+                    qDebug() << "RefreshSchedule: Processing account" << account->accountDisplayString() << "with internal ID"
                              << accountId;
                     return;
                 }
             }
         }
-        qDebug() << "RefreshSchedule: Account with with internal ID " << accountId << " not found.";
+        qDebug() << "RefreshSchedule: Account with internal ID" << accountId << "not found.";
     }
     // if we get here, no account needed refreshing. Schedule refresh in an hour.
     m_refreshTimer->start(1000 * 3600);
@@ -663,7 +647,7 @@ void AccountList::authSucceeded()
 
 void AccountList::authFailed(QString reason)
 {
-    qDebug() << "RefreshSchedule: Background account refresh failed: " << reason;
+    qDebug() << "RefreshSchedule: Background account refresh failed:" << reason;
     m_currentTask.reset();
     m_nextTimer->start(1000 * 20);
 }
@@ -685,7 +669,7 @@ void AccountList::beginActivity()
 void AccountList::endActivity()
 {
     if (m_activityCount == 0) {
-        qWarning() << m_name << " - Activity count would become below zero";
+        qWarning() << "Activity count would become below zero";
         return;
     }
     bool deactivating = m_activityCount == 1;
