@@ -11,20 +11,24 @@
 #include <QStyle>
 #include <QThreadPool>
 #include <QUrl>
+#include <utility>
 
 #include "Application.h"
 #include "FileSystem.h"
 
-#include "QVariantUtils.h"
-#include "StringUtils.h"
-#include "minecraft/mod/tasks/BasicFolderLoadTask.h"
+#include "minecraft/mod/tasks/ResourceFolderLoadTask.h"
 
+#include "Json.h"
+#include "minecraft/mod/tasks/LocalResourceUpdateTask.h"
+#include "modplatform/flame/FlameAPI.h"
+#include "modplatform/flame/FlameModIndex.h"
 #include "settings/Setting.h"
+#include "tasks/SequentialTask.h"
 #include "tasks/Task.h"
 #include "ui/dialogs/CustomMessageBox.h"
 
-ResourceFolderModel::ResourceFolderModel(QDir dir, BaseInstance* instance, QObject* parent, bool create_dir)
-    : QAbstractListModel(parent), m_dir(dir), m_instance(instance), m_watcher(this)
+ResourceFolderModel::ResourceFolderModel(const QDir& dir, BaseInstance* instance, bool is_indexed, bool create_dir, QObject* parent)
+    : QAbstractListModel(parent), m_dir(dir), m_instance(instance), m_watcher(this), m_is_indexed(is_indexed)
 {
     if (create_dir) {
         FS::ensureFolderPathExists(m_dir.absolutePath());
@@ -34,9 +38,12 @@ ResourceFolderModel::ResourceFolderModel(QDir dir, BaseInstance* instance, QObje
     m_dir.setSorting(QDir::Name | QDir::IgnoreCase | QDir::LocaleAware);
 
     connect(&m_watcher, &QFileSystemWatcher::directoryChanged, this, &ResourceFolderModel::directoryChanged);
-    connect(&m_helper_thread_task, &ConcurrentTask::finished, this, [this] { m_helper_thread_task.clear(); });
+    connect(&m_resourceResolver, &ConcurrentTask::finished, this, [this] {
+        m_resourceResolver.clear();
+        m_resourceResolverRunning = false;
+    });
     if (APPLICATION_DYN) {  // in tests the application macro doesn't work
-        m_helper_thread_task.setMaxConcurrent(APPLICATION->settings()->get("NumberOfConcurrentTasks").toInt());
+        m_resourceResolver.setMaxConcurrent(APPLICATION->settings()->get("NumberOfConcurrentTasks").toInt());
     }
 }
 
@@ -48,15 +55,18 @@ ResourceFolderModel::~ResourceFolderModel()
 
 bool ResourceFolderModel::startWatching(const QStringList& paths)
 {
+    // Remove orphaned metadata next time
+    m_first_folder_load = true;
+
     if (m_is_watching)
         return false;
 
     auto couldnt_be_watched = m_watcher.addPaths(paths);
     for (auto path : paths) {
         if (couldnt_be_watched.contains(path))
-            qDebug() << "Failed to start watching " << path;
+            qDebug() << "Failed to start watching" << path;
         else
-            qDebug() << "Started watching " << path;
+            qDebug() << "Started watching" << path;
     }
 
     update();
@@ -73,9 +83,9 @@ bool ResourceFolderModel::stopWatching(const QStringList& paths)
     auto couldnt_be_stopped = m_watcher.removePaths(paths);
     for (auto path : paths) {
         if (couldnt_be_stopped.contains(path))
-            qDebug() << "Failed to stop watching " << path;
+            qDebug() << "Failed to stop watching" << path;
         else
-            qDebug() << "Stopped watching " << path;
+            qDebug() << "Stopped watching" << path;
     }
 
     m_is_watching = !m_is_watching;
@@ -92,7 +102,7 @@ bool ResourceFolderModel::installResource(QString original_path)
         qWarning() << "Caught attempt to install non-existing file or file-like object:" << original_path;
         return false;
     }
-    qDebug() << "Installing: " << file_info.absoluteFilePath();
+    qDebug() << "Installing:" << file_info.absoluteFilePath();
 
     Resource resource(file_info);
     if (!resource.valid()) {
@@ -158,11 +168,57 @@ bool ResourceFolderModel::installResource(QString original_path)
     return false;
 }
 
-bool ResourceFolderModel::uninstallResource(QString file_name)
+void ResourceFolderModel::installResourceWithFlameMetadata(QString path, ModPlatform::IndexedVersion& vers)
+{
+    auto install = [this, path] { installResource(std::move(path)); };
+    if (vers.addonId.isValid()) {
+        ModPlatform::IndexedPack pack{
+            vers.addonId,
+            ModPlatform::ResourceProvider::FLAME,
+        };
+
+        auto response = std::make_shared<QByteArray>();
+        auto job = FlameAPI().getProject(vers.addonId.toString(), response);
+        connect(job.get(), &Task::failed, this, install);
+        connect(job.get(), &Task::aborted, this, install);
+        connect(job.get(), &Task::succeeded, [response, this, &vers, install, &pack] {
+            QJsonParseError parse_error{};
+            QJsonDocument doc = QJsonDocument::fromJson(*response, &parse_error);
+            if (parse_error.error != QJsonParseError::NoError) {
+                qWarning() << "Error while parsing JSON response for mod info at" << parse_error.offset
+                           << "reason:" << parse_error.errorString();
+                qDebug() << *response;
+                return;
+            }
+            try {
+                auto obj = Json::requireObject(Json::requireObject(doc), "data");
+                FlameMod::loadIndexedPack(pack, obj);
+            } catch (const JSONValidationError& e) {
+                qDebug() << doc;
+                qWarning() << "Error while reading mod info:" << e.cause();
+            }
+            LocalResourceUpdateTask update_metadata(indexDir(), pack, vers);
+            connect(&update_metadata, &Task::finished, this, install);
+            update_metadata.start();
+        });
+
+        job->start();
+    } else {
+        install();
+    }
+}
+
+bool ResourceFolderModel::uninstallResource(const QString& file_name, bool preserve_metadata)
 {
     for (auto& resource : m_resources) {
-        if (resource->fileinfo().fileName() == file_name) {
-            auto res = resource->destroy(false);
+        auto resourceFileInfo = resource->fileinfo();
+        auto resourceFileName = resource->fileinfo().fileName();
+        if (!resource->enabled() && resourceFileName.endsWith(".disabled")) {
+            resourceFileName.chop(9);
+        }
+
+        if (resourceFileName == file_name) {
+            auto res = resource->destroy(indexDir(), preserve_metadata, false);
 
             update();
 
@@ -178,13 +234,11 @@ bool ResourceFolderModel::deleteResources(const QModelIndexList& indexes)
         return true;
 
     for (auto i : indexes) {
-        if (i.column() != 0) {
+        if (i.column() != 0)
             continue;
-        }
 
         auto& resource = m_resources.at(i.row());
-
-        resource->destroy();
+        resource->destroy(indexDir());
     }
 
     update();
@@ -192,8 +246,36 @@ bool ResourceFolderModel::deleteResources(const QModelIndexList& indexes)
     return true;
 }
 
+void ResourceFolderModel::deleteMetadata(const QModelIndexList& indexes)
+{
+    if (indexes.isEmpty())
+        return;
+
+    for (auto i : indexes) {
+        if (i.column() != 0)
+            continue;
+
+        auto& resource = m_resources.at(i.row());
+        resource->destroyMetadata(indexDir());
+    }
+
+    update();
+}
+
 bool ResourceFolderModel::setResourceEnabled(const QModelIndexList& indexes, EnableAction action)
 {
+    if (m_instance != nullptr && m_instance->isRunning()) {
+        auto response =
+            CustomMessageBox::selectable(nullptr, tr("Confirm toggle"),
+                                         tr("If you enable/disable this resource while the game is running it may crash your game.\n"
+                                            "Are you sure you want to do this?"),
+                                         QMessageBox::Warning, QMessageBox::Yes | QMessageBox::No, QMessageBox::No)
+                ->exec();
+
+        if (response != QMessageBox::Yes)
+            return false;
+    }
+
     if (indexes.isEmpty())
         return true;
 
@@ -245,7 +327,7 @@ bool ResourceFolderModel::update()
     connect(m_current_update_task.get(), &Task::failed, this, &ResourceFolderModel::onUpdateFailed, Qt::ConnectionType::QueuedConnection);
     connect(
         m_current_update_task.get(), &Task::finished, this,
-        [=] {
+        [this] {
             m_current_update_task.reset();
             if (m_scheduled_update) {
                 m_scheduled_update = false;
@@ -256,7 +338,20 @@ bool ResourceFolderModel::update()
         },
         Qt::ConnectionType::QueuedConnection);
 
-    QThreadPool::globalInstance()->start(m_current_update_task.get());
+    Task::Ptr preUpdate{ createPreUpdateTask() };
+
+    if (preUpdate != nullptr) {
+        auto task = new SequentialTask("ResourceFolderModel::update");
+
+        task->addTask(preUpdate);
+        task->addTask(m_current_update_task);
+
+        connect(task, &Task::finished, [task] { task->deleteLater(); });
+
+        QThreadPool::globalInstance()->start(task);
+    } else {
+        QThreadPool::globalInstance()->start(m_current_update_task.get());
+    }
 
     return true;
 }
@@ -290,29 +385,25 @@ void ResourceFolderModel::resolveResource(Resource::Ptr res)
         },
         Qt::ConnectionType::QueuedConnection);
 
-    m_helper_thread_task.addTask(task);
+    m_resourceResolver.addTask(task);
 
-    if (!m_helper_thread_task.isRunning()) {
-        QThreadPool::globalInstance()->start(&m_helper_thread_task);
+    if (!m_resourceResolverRunning) {
+        QThreadPool::globalInstance()->start(&m_resourceResolver);
+        m_resourceResolverRunning = true;
     }
 }
 
 void ResourceFolderModel::onUpdateSucceeded()
 {
-    auto update_results = static_cast<BasicFolderLoadTask*>(m_current_update_task.get())->result();
+    auto update_results = static_cast<ResourceFolderLoadTask*>(m_current_update_task.get())->result();
 
     auto& new_resources = update_results->resources;
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
     auto current_list = m_resources_index.keys();
     QSet<QString> current_set(current_list.begin(), current_list.end());
 
     auto new_list = new_resources.keys();
     QSet<QString> new_set(new_list.begin(), new_list.end());
-#else
-    QSet<QString> current_set(m_resources_index.keys().toSet());
-    QSet<QString> new_set(new_resources.keys().toSet());
-#endif
 
     applyUpdates(current_set, new_set, new_resources);
 }
@@ -329,7 +420,11 @@ void ResourceFolderModel::onParseSucceeded(int ticket, QString resource_id)
 
 Task* ResourceFolderModel::createUpdateTask()
 {
-    return new BasicFolderLoadTask(m_dir);
+    auto index_dir = indexDir();
+    auto task = new ResourceFolderLoadTask(dir(), index_dir, m_is_indexed, m_first_folder_load,
+                                           [this](const QFileInfo& file) { return createResource(file); });
+    m_first_folder_load = false;
+    return task;
 }
 
 bool ResourceFolderModel::hasPendingParseTasks() const
@@ -419,6 +514,8 @@ QVariant ResourceFolderModel::data(const QModelIndex& index, int role) const
                     return m_resources[row]->name();
                 case DateColumn:
                     return m_resources[row]->dateTimeChanged();
+                case ProviderColumn:
+                    return m_resources[row]->provider();
                 case SizeColumn:
                     return m_resources[row]->sizeStr();
                 default:
@@ -442,17 +539,14 @@ QVariant ResourceFolderModel::data(const QModelIndex& index, int role) const
             return m_resources[row]->internal_id();
         case Qt::DecorationRole: {
             if (column == NameColumn && (at(row).isSymLinkUnder(instDirPath()) || at(row).isMoreThanOneHardLink()))
-                return APPLICATION->getThemedIcon("status-yellow");
+                return QIcon::fromTheme("status-yellow");
 
             return {};
         }
         case Qt::CheckStateRole:
-            switch (column) {
-                case ActiveColumn:
-                    return m_resources[row]->enabled() ? Qt::Checked : Qt::Unchecked;
-                default:
-                    return {};
-            }
+            if (column == ActiveColumn)
+                return m_resources[row]->enabled() ? Qt::Checked : Qt::Unchecked;
+            return {};
         default:
             return {};
     }
@@ -465,17 +559,6 @@ bool ResourceFolderModel::setData(const QModelIndex& index, [[maybe_unused]] con
         return false;
 
     if (role == Qt::CheckStateRole) {
-        if (m_instance != nullptr && m_instance->isRunning()) {
-            auto response =
-                CustomMessageBox::selectable(nullptr, tr("Confirm toggle"),
-                                             tr("If you enable/disable this resource while the game is running it may crash your game.\n"
-                                                "Are you sure you want to do this?"),
-                                             QMessageBox::Warning, QMessageBox::Yes | QMessageBox::No, QMessageBox::No)
-                    ->exec();
-
-            if (response != QMessageBox::Yes)
-                return false;
-        }
         return setResourceEnabled({ index }, EnableAction::TOGGLE);
     }
 
@@ -490,22 +573,23 @@ QVariant ResourceFolderModel::headerData(int section, [[maybe_unused]] Qt::Orien
                 case ActiveColumn:
                 case NameColumn:
                 case DateColumn:
+                case ProviderColumn:
                 case SizeColumn:
                     return columnNames().at(section);
                 default:
                     return {};
             }
         case Qt::ToolTipRole: {
+            //: Here, resource is a generic term for external resources, like Mods, Resource Packs, Shader Packs, etc.
             switch (section) {
                 case ActiveColumn:
-                    //: Here, resource is a generic term for external resources, like Mods, Resource Packs, Shader Packs, etc.
                     return tr("Is the resource enabled?");
                 case NameColumn:
-                    //: Here, resource is a generic term for external resources, like Mods, Resource Packs, Shader Packs, etc.
                     return tr("The name of the resource.");
                 case DateColumn:
-                    //: Here, resource is a generic term for external resources, like Mods, Resource Packs, Shader Packs, etc.
                     return tr("The date and time this resource was last changed (or added).");
+                case ProviderColumn:
+                    return tr("The source provider of the resource.");
                 case SizeColumn:
                     return tr("The size of the resource.");
                 default:
@@ -528,30 +612,89 @@ void ResourceFolderModel::setupHeaderAction(QAction* act, int column)
 
 void ResourceFolderModel::saveColumns(QTreeView* tree)
 {
-    auto const setting_name = QString("UI/%1_Page/Columns").arg(id());
-    auto setting = (m_instance->settings()->contains(setting_name)) ? m_instance->settings()->getSetting(setting_name)
-                                                                    : m_instance->settings()->registerSetting(setting_name);
+    auto const stateSettingName = QString("UI/%1_Page/Columns").arg(id());
+    auto const overrideSettingName = QString("UI/%1_Page/ColumnsOverride").arg(id());
+    auto const visibilitySettingName = QString("UI/%1_Page/ColumnsVisibility").arg(id());
 
-    setting->set(tree->header()->saveState());
+    auto stateSetting = m_instance->settings()->getSetting(stateSettingName);
+    stateSetting->set(QString::fromUtf8(tree->header()->saveState().toBase64()));
+
+    // neither passthrough nor override settings works for this usecase as I need to only set the global when the gate is false
+    auto settings = m_instance->settings();
+    if (!settings->get(overrideSettingName).toBool()) {
+        settings = APPLICATION->settings();
+    }
+    auto visibility = Json::toMap(settings->get(visibilitySettingName).toString());
+    for (auto i = 0; i < m_column_names.size(); ++i) {
+        if (m_columnsHideable[i]) {
+            auto name = m_column_names[i];
+            visibility[name] = !tree->isColumnHidden(i);
+        }
+    }
+    settings->set(visibilitySettingName, Json::fromMap(visibility));
 }
 
 void ResourceFolderModel::loadColumns(QTreeView* tree)
 {
-    for (auto i = 0; i < m_columnsHiddenByDefault.size(); ++i) {
-        tree->setColumnHidden(i, m_columnsHiddenByDefault[i]);
+    auto const stateSettingName = QString("UI/%1_Page/Columns").arg(id());
+    auto const overrideSettingName = QString("UI/%1_Page/ColumnsOverride").arg(id());
+    auto const visibilitySettingName = QString("UI/%1_Page/ColumnsVisibility").arg(id());
+
+    auto stateSetting = m_instance->settings()->getOrRegisterSetting(stateSettingName, "");
+    tree->header()->restoreState(QByteArray::fromBase64(stateSetting->get().toString().toUtf8()));
+
+    auto setVisible = [this, tree](QVariant value) {
+        auto visibility = Json::toMap(value.toString());
+        for (auto i = 0; i < m_column_names.size(); ++i) {
+            if (m_columnsHideable[i]) {
+                auto name = m_column_names[i];
+                tree->setColumnHidden(i, !visibility.value(name, false).toBool());
+            }
+        }
+    };
+
+    auto const defaultValue = Json::fromMap({
+        { "Image", true },
+        { "Version", true },
+        { "Last Modified", true },
+        { "Provider", true },
+        { "Pack Format", true },
+    });
+    // neither passthrough nor override settings works for this usecase as I need to only set the global when the gate is false
+    auto settings = m_instance->settings();
+    if (!settings->getOrRegisterSetting(overrideSettingName, false)->get().toBool()) {
+        settings = APPLICATION->settings();
     }
+    auto visibility = settings->getOrRegisterSetting(visibilitySettingName, defaultValue);
+    setVisible(visibility->get());
 
-    auto const setting_name = QString("UI/%1_Page/Columns").arg(id());
-    auto setting = (m_instance->settings()->contains(setting_name)) ? m_instance->settings()->getSetting(setting_name)
-                                                                    : m_instance->settings()->registerSetting(setting_name);
-
-    tree->header()->restoreState(setting->get().toByteArray());
+    // allways connect the signal in case the setting is toggled on and off
+    auto gSetting = APPLICATION->settings()->getOrRegisterSetting(visibilitySettingName, defaultValue);
+    connect(gSetting.get(), &Setting::SettingChanged, tree, [this, setVisible, overrideSettingName](const Setting&, QVariant value) {
+        if (!m_instance->settings()->get(overrideSettingName).toBool()) {
+            setVisible(value);
+        }
+    });
 }
 
 QMenu* ResourceFolderModel::createHeaderContextMenu(QTreeView* tree)
 {
     auto menu = new QMenu(tree);
 
+    {  // action to decide if the visibility is per instance or not
+        auto act = new QAction(tr("Override Columns Visibility"), menu);
+        auto const overrideSettingName = QString("UI/%1_Page/ColumnsOverride").arg(id());
+
+        act->setCheckable(true);
+        act->setChecked(m_instance->settings()->getOrRegisterSetting(overrideSettingName, false)->get().toBool());
+
+        connect(act, &QAction::toggled, tree, [this, tree, overrideSettingName](bool toggled) {
+            m_instance->settings()->set(overrideSettingName, toggled);
+            saveColumns(tree);
+        });
+
+        menu->addAction(act);
+    }
     menu->addSeparator()->setText(tr("Show / Hide Columns"));
 
     for (int col = 0; col < columnCount(); ++col) {
@@ -591,8 +734,7 @@ SortType ResourceFolderModel::columnToSortKey(size_t column) const
 }
 
 /* Standard Proxy Model for createFilterProxyModel */
-[[nodiscard]] bool ResourceFolderModel::ProxyModel::filterAcceptsRow(int source_row,
-                                                                     [[maybe_unused]] const QModelIndex& source_parent) const
+bool ResourceFolderModel::ProxyModel::filterAcceptsRow(int source_row, [[maybe_unused]] const QModelIndex& source_parent) const
 {
     auto* model = qobject_cast<ResourceFolderModel*>(sourceModel());
     if (!model)
@@ -603,7 +745,7 @@ SortType ResourceFolderModel::columnToSortKey(size_t column) const
     return resource.applyFilter(filterRegularExpression());
 }
 
-[[nodiscard]] bool ResourceFolderModel::ProxyModel::lessThan(const QModelIndex& source_left, const QModelIndex& source_right) const
+bool ResourceFolderModel::ProxyModel::lessThan(const QModelIndex& source_left, const QModelIndex& source_right) const
 {
     auto* model = qobject_cast<ResourceFolderModel*>(sourceModel());
     if (!model || !source_left.isValid() || !source_right.isValid() || source_left.column() != source_right.column()) {
@@ -651,3 +793,126 @@ void ResourceFolderModel::onParseFailed(int ticket, QString resource_id)
     }
     endRemoveRows();
 }
+
+void ResourceFolderModel::applyUpdates(QSet<QString>& current_set, QSet<QString>& new_set, QMap<QString, Resource::Ptr>& new_resources)
+{
+    // see if the kept resources changed in some way
+    {
+        QSet<QString> kept_set = current_set;
+        kept_set.intersect(new_set);
+
+        for (auto const& kept : kept_set) {
+            auto row_it = m_resources_index.constFind(kept);
+            Q_ASSERT(row_it != m_resources_index.constEnd());
+            auto row = row_it.value();
+
+            auto& new_resource = new_resources[kept];
+            auto const& current_resource = m_resources.at(row);
+
+            if (new_resource->dateTimeChanged() == current_resource->dateTimeChanged()) {
+                // no significant change, ignore...
+                continue;
+            }
+
+            // If the resource is resolving, but something about it changed, we don't want to
+            // continue the resolving.
+            if (current_resource->isResolving()) {
+                auto ticket = current_resource->resolutionTicket();
+                if (m_active_parse_tasks.contains(ticket)) {
+                    auto task = (*m_active_parse_tasks.find(ticket)).get();
+                    task->abort();
+                }
+            }
+
+            m_resources[row].reset(new_resource);
+            resolveResource(m_resources.at(row));
+            emit dataChanged(index(row, 0), index(row, columnCount(QModelIndex()) - 1));
+        }
+    }
+
+    // remove resources no longer present
+    {
+        QSet<QString> removed_set = current_set;
+        removed_set.subtract(new_set);
+
+        QList<int> removed_rows;
+        for (auto& removed : removed_set)
+            removed_rows.append(m_resources_index[removed]);
+
+        std::sort(removed_rows.begin(), removed_rows.end(), std::greater<int>());
+
+        for (auto& removed_index : removed_rows) {
+            auto removed_it = m_resources.begin() + removed_index;
+
+            Q_ASSERT(removed_it != m_resources.end());
+
+            if ((*removed_it)->isResolving()) {
+                auto ticket = (*removed_it)->resolutionTicket();
+                if (m_active_parse_tasks.contains(ticket)) {
+                    auto task = (*m_active_parse_tasks.find(ticket)).get();
+                    task->abort();
+                }
+            }
+
+            beginRemoveRows(QModelIndex(), removed_index, removed_index);
+            m_resources.erase(removed_it);
+            endRemoveRows();
+        }
+    }
+
+    // add new resources to the end
+    {
+        QSet<QString> added_set = new_set;
+        added_set.subtract(current_set);
+
+        // When you have a Qt build with assertions turned on, proceeding here will abort the application
+        if (added_set.size() > 0) {
+            beginInsertRows(QModelIndex(), static_cast<int>(m_resources.size()),
+                            static_cast<int>(m_resources.size() + added_set.size() - 1));
+
+            for (auto& added : added_set) {
+                auto res = new_resources[added];
+                m_resources.append(res);
+                resolveResource(m_resources.last());
+            }
+
+            endInsertRows();
+        }
+    }
+
+    // update index
+    {
+        m_resources_index.clear();
+        int idx = 0;
+        for (auto const& mod : qAsConst(m_resources)) {
+            m_resources_index[mod->internal_id()] = idx;
+            idx++;
+        }
+    }
+}
+Resource::Ptr ResourceFolderModel::find(QString id)
+{
+    auto iter =
+        std::find_if(m_resources.constBegin(), m_resources.constEnd(), [&](Resource::Ptr const& r) { return r->internal_id() == id; });
+    if (iter == m_resources.constEnd())
+        return nullptr;
+    return *iter;
+}
+QList<Resource*> ResourceFolderModel::allResources()
+{
+    QList<Resource*> result;
+    result.reserve(m_resources.size());
+    for (const Resource ::Ptr& resource : m_resources)
+        result.append((resource.get()));
+    return result;
+}
+QList<Resource*> ResourceFolderModel::selectedResources(const QModelIndexList& indexes)
+{
+    QList<Resource*> result;
+    for (const QModelIndex& index : indexes) {
+        if (index.column() != 0)
+            continue;
+        result.append(&at(index.row()));
+    }
+    return result;
+}
