@@ -39,8 +39,6 @@
 #include <QtConcurrent>
 #include <algorithm>
 
-#include <quazip/quazip.h>
-
 #include "FileSystem.h"
 #include "Json.h"
 #include "MMCZip.h"
@@ -69,7 +67,8 @@ PackInstallTask::PackInstallTask(UserInteractionSupport* support, QString packNa
 {
     m_support = support;
     m_pack_name = packName;
-    m_pack_safe_name = packName.replace(QRegularExpression("[^A-Za-z0-9]"), "");
+    static const QRegularExpression s_regex("[^A-Za-z0-9]");
+    m_pack_safe_name = packName.replace(s_regex, "");
     m_version_name = version;
     m_install_mode = installMode;
 }
@@ -84,15 +83,15 @@ bool PackInstallTask::abort()
 
 void PackInstallTask::executeTask()
 {
-    qDebug() << "PackInstallTask::executeTask: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::executeTask:" << QThread::currentThreadId();
     NetJob::Ptr netJob{ new NetJob("ATLauncher::VersionFetch", APPLICATION->network()) };
     auto searchUrl =
         QString(BuildConfig.ATL_DOWNLOAD_SERVER_URL + "packs/%1/versions/%2/Configs.json").arg(m_pack_safe_name).arg(m_version_name);
     netJob->addNetAction(Net::ApiDownload::makeByteArray(QUrl(searchUrl), response));
 
-    QObject::connect(netJob.get(), &NetJob::succeeded, this, &PackInstallTask::onDownloadSucceeded);
-    QObject::connect(netJob.get(), &NetJob::failed, this, &PackInstallTask::onDownloadFailed);
-    QObject::connect(netJob.get(), &NetJob::aborted, this, &PackInstallTask::onDownloadAborted);
+    connect(netJob.get(), &NetJob::succeeded, this, &PackInstallTask::onDownloadSucceeded);
+    connect(netJob.get(), &NetJob::failed, this, &PackInstallTask::onDownloadFailed);
+    connect(netJob.get(), &NetJob::aborted, this, &PackInstallTask::onDownloadAborted);
 
     jobPtr = netJob;
     jobPtr->start();
@@ -100,14 +99,14 @@ void PackInstallTask::executeTask()
 
 void PackInstallTask::onDownloadSucceeded()
 {
-    qDebug() << "PackInstallTask::onDownloadSucceeded: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::onDownloadSucceeded:" << QThread::currentThreadId();
     jobPtr.reset();
 
     QJsonParseError parse_error{};
     QJsonDocument doc = QJsonDocument::fromJson(*response, &parse_error);
     if (parse_error.error != QJsonParseError::NoError) {
-        qWarning() << "Error while parsing JSON response from ATLauncher at " << parse_error.offset
-                   << " reason: " << parse_error.errorString();
+        qWarning() << "Error while parsing JSON response from ATLauncher at" << parse_error.offset
+                   << "reason:" << parse_error.errorString();
         qWarning() << *response.get();
         return;
     }
@@ -167,7 +166,7 @@ void PackInstallTask::onDownloadSucceeded()
 
 void PackInstallTask::onDownloadFailed(QString reason)
 {
-    qDebug() << "PackInstallTask::onDownloadFailed: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::onDownloadFailed:" << QThread::currentThreadId();
     jobPtr.reset();
     emitFailed(reason);
 }
@@ -390,7 +389,7 @@ QString PackInstallTask::getVersionForLoader(QString uid)
     return m_version.loader.version;
 }
 
-QString PackInstallTask::detectLibrary(VersionLibrary library)
+QString PackInstallTask::detectLibrary(const VersionLibrary& library)
 {
     // Try to detect what the library is
     if (!library.server.isEmpty() && library.server.split("/").length() >= 3) {
@@ -433,14 +432,15 @@ bool PackInstallTask::createLibrariesComponent(QString instanceRoot, std::shared
     QList<GradleSpecifier> exempt;
     for (const auto& componentUid : componentsToInstall.keys()) {
         auto componentVersion = componentsToInstall.value(componentUid);
-
-        for (const auto& library : componentVersion->data()->libraries) {
-            GradleSpecifier lib(library->rawName());
-            exempt.append(lib);
+        if (componentVersion->data()) {
+            for (const auto& library : componentVersion->data()->libraries) {
+                GradleSpecifier lib(library->rawName());
+                exempt.append(lib);
+            }
         }
     }
 
-    {
+    if (minecraftVersion->data()) {
         for (const auto& library : minecraftVersion->data()->libraries) {
             GradleSpecifier lib(library->rawName());
             exempt.append(lib);
@@ -581,10 +581,12 @@ bool PackInstallTask::createPackComponent(QString instanceRoot, std::shared_ptr<
     for (const auto& componentUid : componentsToInstall.keys()) {
         auto componentVersion = componentsToInstall.value(componentUid);
 
-        if (componentVersion->data()->mainClass != QString("")) {
-            mainClasses.append(componentVersion->data()->mainClass);
+        if (componentVersion->data()) {
+            if (componentVersion->data()->mainClass != QString("")) {
+                mainClasses.append(componentVersion->data()->mainClass);
+            }
+            tweakers.append(componentVersion->data()->addTweakers);
         }
-        tweakers.append(componentVersion->data()->addTweakers);
     }
 
     auto f = std::make_shared<VersionFile>();
@@ -625,7 +627,7 @@ bool PackInstallTask::createPackComponent(QString instanceRoot, std::shared_ptr<
 
 void PackInstallTask::installConfigs()
 {
-    qDebug() << "PackInstallTask::installConfigs: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::installConfigs:" << QThread::currentThreadId();
     setStatus(tr("Downloading configs..."));
     jobPtr.reset(new NetJob(tr("Config download"), APPLICATION->network()));
 
@@ -641,22 +643,22 @@ void PackInstallTask::installConfigs()
     jobPtr->addNetAction(dl);
     archivePath = entry->getFullPath();
 
-    connect(jobPtr.get(), &NetJob::succeeded, this, [&]() {
+    connect(jobPtr.get(), &NetJob::succeeded, this, [this]() {
         abortable = false;
         jobPtr.reset();
         extractConfigs();
     });
-    connect(jobPtr.get(), &NetJob::failed, [&](QString reason) {
+    connect(jobPtr.get(), &NetJob::failed, [this](QString reason) {
         abortable = false;
         jobPtr.reset();
         emitFailed(reason);
     });
-    connect(jobPtr.get(), &NetJob::progress, [&](qint64 current, qint64 total) {
+    connect(jobPtr.get(), &NetJob::progress, [this](qint64 current, qint64 total) {
         abortable = true;
         setProgress(current, total);
     });
     connect(jobPtr.get(), &NetJob::stepProgress, this, &PackInstallTask::propagateStepProgress);
-    connect(jobPtr.get(), &NetJob::aborted, [&] {
+    connect(jobPtr.get(), &NetJob::aborted, [this] {
         abortable = false;
         jobPtr.reset();
         emitAborted();
@@ -667,34 +669,22 @@ void PackInstallTask::installConfigs()
 
 void PackInstallTask::extractConfigs()
 {
-    qDebug() << "PackInstallTask::extractConfigs: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::extractConfigs:" << QThread::currentThreadId();
     setStatus(tr("Extracting configs..."));
 
     QDir extractDir(m_stagingPath);
-
-    QuaZip packZip(archivePath);
-    if (!packZip.open(QuaZip::mdUnzip)) {
-        emitFailed(tr("Failed to open pack configs %1!").arg(archivePath));
-        return;
-    }
-
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     m_extractFuture = QtConcurrent::run(QThreadPool::globalInstance(), QOverload<QString, QString>::of(MMCZip::extractDir), archivePath,
                                         extractDir.absolutePath() + "/minecraft");
-#else
-    m_extractFuture =
-        QtConcurrent::run(QThreadPool::globalInstance(), MMCZip::extractDir, archivePath, extractDir.absolutePath() + "/minecraft");
-#endif
-    connect(&m_extractFutureWatcher, &QFutureWatcher<QStringList>::finished, this, [&]() { downloadMods(); });
-    connect(&m_extractFutureWatcher, &QFutureWatcher<QStringList>::canceled, this, [&]() { emitAborted(); });
+    connect(&m_extractFutureWatcher, &QFutureWatcher<QStringList>::finished, this, [this]() { downloadMods(); });
+    connect(&m_extractFutureWatcher, &QFutureWatcher<QStringList>::canceled, this, [this]() { emitAborted(); });
     m_extractFutureWatcher.setFuture(m_extractFuture);
 }
 
 void PackInstallTask::downloadMods()
 {
-    qDebug() << "PackInstallTask::installMods: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::installMods:" << QThread::currentThreadId();
 
-    QVector<ATLauncher::VersionMod> optionalMods;
+    QList<ATLauncher::VersionMod> optionalMods;
     for (const auto& mod : m_version.mods) {
         if (mod.optional) {
             optionalMods.push_back(mod);
@@ -702,7 +692,7 @@ void PackInstallTask::downloadMods()
     }
 
     // Select optional mods, if pack contains any
-    QVector<QString> selectedMods;
+    QList<QString> selectedMods;
     if (!optionalMods.isEmpty()) {
         setStatus(tr("Selecting optional mods..."));
         auto mods = m_support->chooseOptionalMods(m_version, optionalMods);
@@ -829,7 +819,7 @@ void PackInstallTask::downloadMods()
         message_dialog.setModal(true);
 
         if (message_dialog.exec()) {
-            qDebug() << "Post dialog blocked mods list: " << mods;
+            qDebug() << "Post dialog blocked mods list:" << mods;
             for (auto blocked : mods) {
                 if (!blocked.matched) {
                     qDebug() << blocked.name << "was not matched to a local file, skipping copy";
@@ -893,17 +883,12 @@ void PackInstallTask::onModsDownloaded()
 {
     abortable = false;
 
-    qDebug() << "PackInstallTask::onModsDownloaded: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::onModsDownloaded:" << QThread::currentThreadId();
     jobPtr.reset();
 
     if (!modsToExtract.empty() || !modsToDecomp.empty() || !modsToCopy.empty()) {
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
         m_modExtractFuture =
             QtConcurrent::run(QThreadPool::globalInstance(), &PackInstallTask::extractMods, this, modsToExtract, modsToDecomp, modsToCopy);
-#else
-        m_modExtractFuture =
-            QtConcurrent::run(QThreadPool::globalInstance(), this, &PackInstallTask::extractMods, modsToExtract, modsToDecomp, modsToCopy);
-#endif
         connect(&m_modExtractFutureWatcher, &QFutureWatcher<QStringList>::finished, this, &PackInstallTask::onModsExtracted);
         connect(&m_modExtractFutureWatcher, &QFutureWatcher<QStringList>::canceled, this, &PackInstallTask::emitAborted);
         m_modExtractFutureWatcher.setFuture(m_modExtractFuture);
@@ -914,7 +899,7 @@ void PackInstallTask::onModsDownloaded()
 
 void PackInstallTask::onModsExtracted()
 {
-    qDebug() << "PackInstallTask::onModsExtracted: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::onModsExtracted:" << QThread::currentThreadId();
     if (m_modExtractFuture.result()) {
         install();
     } else {
@@ -926,7 +911,7 @@ bool PackInstallTask::extractMods(const QMap<QString, VersionMod>& toExtract,
                                   const QMap<QString, VersionMod>& toDecomp,
                                   const QMap<QString, QString>& toCopy)
 {
-    qDebug() << "PackInstallTask::extractMods: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::extractMods:" << QThread::currentThreadId();
 
     setStatus(tr("Extracting mods..."));
     for (auto iter = toExtract.begin(); iter != toExtract.end(); iter++) {
@@ -948,7 +933,8 @@ bool PackInstallTask::extractMods(const QMap<QString, VersionMod>& toExtract,
         QString folderToExtract = "";
         if (mod.type == ModType::Extract) {
             folderToExtract = mod.extractFolder;
-            folderToExtract.remove(QRegularExpression("^/"));
+            static const QRegularExpression s_regex("^/");
+            folderToExtract.remove(s_regex);
         }
 
         qDebug() << "Extracting " + mod.file + " to " + extractToDir;
@@ -998,7 +984,7 @@ bool PackInstallTask::extractMods(const QMap<QString, VersionMod>& toExtract,
 
 void PackInstallTask::install()
 {
-    qDebug() << "PackInstallTask::install: " << QThread::currentThreadId();
+    qDebug() << "PackInstallTask::install:" << QThread::currentThreadId();
     setStatus(tr("Installing modpack"));
 
     auto instanceConfigPath = FS::PathCombine(m_stagingPath, "instance.cfg");
