@@ -20,7 +20,6 @@
 #include <algorithm>
 
 #include "Json.h"
-#include "QObjectPtr.h"
 #include "modplatform/ModIndex.h"
 #include "modplatform/flame/FlameAPI.h"
 #include "modplatform/flame/FlameModIndex.h"
@@ -33,9 +32,7 @@
 static const FlameAPI flameAPI;
 static ModrinthAPI modrinthAPI;
 
-Flame::FileResolvingTask::FileResolvingTask(const shared_qobject_ptr<QNetworkAccessManager>& network, Flame::Manifest& toProcess)
-    : m_network(network), m_manifest(toProcess)
-{}
+Flame::FileResolvingTask::FileResolvingTask(Flame::Manifest& toProcess) : m_manifest(toProcess) {}
 
 bool Flame::FileResolvingTask::abort()
 {
@@ -87,6 +84,30 @@ void Flame::FileResolvingTask::executeTask()
     m_task->start();
 }
 
+ModPlatform::ResourceType getResourceType(int classId)
+{
+    switch (classId) {
+        case 17:  // Worlds
+            return ModPlatform::ResourceType::World;
+        case 6:  // Mods
+            return ModPlatform::ResourceType::Mod;
+        case 12:  // Resource Packs
+                  // return ModPlatform::ResourceType::ResourcePack; // not really a resourcepack
+            /* fallthrough */
+        case 4546:  // Customization
+                    // return ModPlatform::ResourceType::ShaderPack; // not really a shaderPack
+            /* fallthrough */
+        case 4471:  // Modpacks
+            /* fallthrough */
+        case 5:  // Bukkit Plugins
+            /* fallthrough */
+        case 4559:  // Addons
+            /* fallthrough */
+        default:
+            return ModPlatform::ResourceType::Unknown;
+    }
+}
+
 void Flame::FileResolvingTask::netJobFinished()
 {
     setProgress(1, 3);
@@ -142,8 +163,8 @@ void Flame::FileResolvingTask::netJobFinished()
         QJsonParseError parse_error{};
         QJsonDocument doc = QJsonDocument::fromJson(*m_result, &parse_error);
         if (parse_error.error != QJsonParseError::NoError) {
-            qWarning() << "Error while parsing JSON response from Modrinth::CurrentVersions at " << parse_error.offset
-                       << " reason: " << parse_error.errorString();
+            qWarning() << "Error while parsing JSON response from Modrinth::CurrentVersions at" << parse_error.offset
+                       << "reason:" << parse_error.errorString();
             qWarning() << *m_result;
 
             getFlameProjects();
@@ -165,7 +186,7 @@ void Flame::FileResolvingTask::netJobFinished()
                         // let the user download it manually.
                         if (!file.loaders || hasSingleModLoaderSelected(file.loaders)) {
                             out.version.downloadUrl = file.downloadUrl;
-                            qDebug() << "Found alternative on modrinth " << out.version.fileName;
+                            qDebug() << "Found alternative on modrinth" << out.version.fileName;
                         }
                     } catch (Json::JsonException& e) {
                         qDebug() << e.cause();
@@ -213,8 +234,8 @@ void Flame::FileResolvingTask::getFlameProjects()
         QJsonParseError parse_error{};
         auto doc = QJsonDocument::fromJson(*m_result, &parse_error);
         if (parse_error.error != QJsonParseError::NoError) {
-            qWarning() << "Error while parsing JSON response from Modrinth projects task at " << parse_error.offset
-                       << " reason: " << parse_error.errorString();
+            qWarning() << "Error while parsing JSON response from Modrinth projects task at" << parse_error.offset
+                       << "reason:" << parse_error.errorString();
             qWarning() << *m_result;
             return;
         }
@@ -234,6 +255,10 @@ void Flame::FileResolvingTask::getFlameProjects()
 
                 setStatus(tr("Parsing API response from CurseForge for '%1'...").arg(file->version.fileName));
                 FlameMod::loadIndexedPack(file->pack, entry_obj);
+                file->resourceType = getResourceType(Json::requireInteger(entry_obj, "classId", "modClassId"));
+                if (file->resourceType == ModPlatform::ResourceType::World) {
+                    file->targetFolder = "saves";
+                }
             }
         } catch (Json::JsonException& e) {
             qDebug() << e.cause();
