@@ -41,8 +41,10 @@
 
 #include <QDateTime>
 #include <QFileInfo>
+#include <QLocale>
 #include <QNetworkReply>
 #include <QUrl>
+#include <cstdint>
 #include <memory>
 
 #if defined(LAUNCHER_APPLICATION)
@@ -55,6 +57,11 @@
 
 namespace Net {
 
+NetRequest::NetRequest() : Task()
+{
+    connect(&m_retryTimer, &QTimer::timeout, this, &NetRequest::executeTask);
+}
+
 void NetRequest::addValidator(Validator* v)
 {
     m_sink->addValidator(v);
@@ -75,16 +82,17 @@ void NetRequest::executeTask()
     m_state = m_sink->init(request);
     switch (m_state) {
         case State::Succeeded:
-            qCDebug(logCat) << getUid().toString() << "Request cache hit " << m_url.toString();
+            qCDebug(logCat) << getUid().toString() << "Request cache hit" << m_url.toString();
             emit succeeded();
             emit finished();
             return;
         case State::Running:
-            qCDebug(logCat) << getUid().toString() << "Running " << m_url.toString();
+            qCDebug(logCat) << getUid().toString() << "Running" << m_url.toString();
             break;
         case State::Inactive:
         case State::Failed:
-            emit failed("Failed to initialize sink");
+            m_failReason = m_sink->failReason();
+            emit failed(m_sink->failReason());
             emit finished();
             return;
         case State::AbortedByUser:
@@ -104,12 +112,10 @@ void NetRequest::executeTask()
         header_proxy->writeHeaders(request);
     }
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)
 #if defined(LAUNCHER_APPLICATION)
     request.setTransferTimeout(APPLICATION->settings()->get("RequestTimeout").toInt() * 1000);
 #else
     request.setTransferTimeout();
-#endif
 #endif
 
     m_last_progress_time = m_clock.now();
@@ -122,11 +128,7 @@ void NetRequest::executeTask()
     connect(rep, &QNetworkReply::uploadProgress, this, &NetRequest::onProgress);
     connect(rep, &QNetworkReply::downloadProgress, this, &NetRequest::onProgress);
     connect(rep, &QNetworkReply::finished, this, &NetRequest::downloadFinished);
-#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)  // QNetworkReply::errorOccurred added in 5.15
     connect(rep, &QNetworkReply::errorOccurred, this, &NetRequest::downloadError);
-#else
-    connect(rep, QOverload<QNetworkReply::NetworkError>::of(&QNetworkReply::error), this, &NetRequest::downloadError);
-#endif
     connect(rep, &QNetworkReply::sslErrors, this, &NetRequest::sslErrors);
     connect(rep, &QNetworkReply::readyRead, this, &NetRequest::downloadReadyRead);
 }
@@ -164,8 +166,22 @@ void NetRequest::onProgress(qint64 bytesReceived, qint64 bytesTotal)
 void NetRequest::downloadError(QNetworkReply::NetworkError error)
 {
     if (error == QNetworkReply::OperationCanceledError) {
-        qCCritical(logCat) << getUid().toString() << "Aborted " << m_url.toString();
+        qCCritical(logCat) << getUid().toString() << "Aborted" << m_url.toString();
         m_state = State::Failed;
+    } else if (replyStatusCode() == 429 /* HTTP Too Many Requests*/ && m_options & Option::AutoRetry) {
+        qCDebug(logCat) << getUid().toString() << "Rate Limited!";
+        int64_t delay = 10 * std::pow(2, m_retryCount);
+        if (m_reply->hasRawHeader("Retry-After")) {
+            auto retryAfter = m_reply->rawHeader("Retry-After");
+            if (retryAfter.trimmed().endsWith("GMT")) /* HTTP Date format */ {
+                auto afterTimestamp = QDateTime::fromString(QString::fromUtf8(retryAfter.trimmed()), "ddd, dd MMM yyyy HH:mm:ss 'GMT'");
+                auto now = QDateTime::currentDateTime();
+                delay = now.secsTo(afterTimestamp);
+            } else {
+                delay = retryAfter.toLong();
+            }
+        }
+        handleAutoRetry(delay);
     } else {
         if (m_options & Option::AcceptLocalFiles) {
             if (m_sink->hasLocalData()) {
@@ -174,9 +190,11 @@ void NetRequest::downloadError(QNetworkReply::NetworkError error)
             }
         }
         // error happened during download.
-        qCCritical(logCat) << getUid().toString() << "Failed" << m_url.toString() << "with reason" << error;
+        qCCritical(logCat) << getUid().toString() << "Failed" << m_url.toString() << "with error" << error;
         if (m_reply)
-            qCCritical(logCat) << getUid().toString() << "HTTP Status" << replyStatusCode() << ";error" << errorString();
+            qCCritical(logCat) << getUid().toString() << "HTTP status:" << replyStatusCode() << errorString();
+        if (m_errorResponse.size() > 0)
+            qCCritical(logCat) << getUid().toString() << "Response from server:" << m_errorResponse;
         m_state = State::Failed;
     }
 }
@@ -185,7 +203,8 @@ void NetRequest::sslErrors(const QList<QSslError>& errors)
 {
     int i = 1;
     for (auto error : errors) {
-        qCCritical(logCat) << getUid().toString() << "Request" << m_url.toString() << "SSL Error #" << i << " : " << error.errorString();
+        qCCritical(logCat).nospace() << getUid().toString() << " Request " << m_url.toString() << " SSL Error #" << i << ": "
+                                     << error.errorString();
         auto cert = error.certificate();
         qCCritical(logCat) << getUid().toString() << "Certificate in question:\n" << cert.toText();
         i++;
@@ -240,14 +259,39 @@ auto NetRequest::handleRedirect() -> bool
     }
 
     m_url = QUrl(redirect.toString());
-    qCDebug(logCat) << getUid().toString() << "Following redirect to " << m_url.toString();
+    qCDebug(logCat) << getUid().toString() << "Following redirect to" << m_url.toString();
     executeTask();
 
     return true;
 }
 
+void NetRequest::handleAutoRetry(int64_t delay)
+{
+    m_retryCount++;
+    if (delay > 60 || m_retryCount > 4) {
+        /* 1 minute is too long to wait for retry, fail for now */
+        m_state = State::Failed;
+        auto retryAfter = QDateTime::currentDateTime().addSecs(delay);
+        emitFailed(tr("Request Rate Limited for %n second(s): Retry After %1", "seconds", delay)
+                       .arg(retryAfter.toLocalTime().toString(QLocale::system().dateTimeFormat(QLocale::ShortFormat))));
+        return;
+    } else {
+        qCDebug(logCat) << getUid().toString() << "Retyring Request in" << delay << "seconds";
+        setStatus(tr("Rate Limited: Waiting %n second(s)", "seconds", delay));
+        m_retryTimer.setTimerType(Qt::VeryCoarseTimer);
+        m_retryTimer.setSingleShot(true);
+        m_retryTimer.setInterval(delay * 1000);
+        m_retryTimer.start();
+    }
+}
+
 void NetRequest::downloadFinished()
 {
+    // currently waiting for retry
+    if (m_retryTimer.isActive()) {
+        return;
+    }
+
     // handle HTTP redirection first
     if (handleRedirect()) {
         qCDebug(logCat) << getUid().toString() << "Request redirected:" << m_url.toString();
@@ -265,6 +309,7 @@ void NetRequest::downloadFinished()
     } else if (m_state == State::Failed) {
         qCDebug(logCat) << getUid().toString() << "Request failed in previous step:" << m_url.toString();
         m_sink->abort();
+        m_failReason = m_reply->errorString();
         emit failed(m_reply->errorString());
         emit finished();
         return;
@@ -284,7 +329,8 @@ void NetRequest::downloadFinished()
         if (m_state != State::Succeeded) {
             qCDebug(logCat) << getUid().toString() << "Request failed to write:" << m_url.toString();
             m_sink->abort();
-            emit failed("failed to write in sink");
+            m_failReason = m_sink->failReason();
+            emit failed(m_sink->failReason());
             emit finished();
             return;
         }
@@ -295,7 +341,8 @@ void NetRequest::downloadFinished()
     if (m_state != State::Succeeded) {
         qCDebug(logCat) << getUid().toString() << "Request failed to finalize:" << m_url.toString();
         m_sink->abort();
-        emit failed("failed to finalize the request");
+        m_failReason = m_sink->failReason();
+        emit failed(m_sink->failReason());
         emit finished();
         return;
     }
@@ -310,12 +357,15 @@ void NetRequest::downloadReadyRead()
     if (m_state == State::Running) {
         auto data = m_reply->readAll();
         m_state = m_sink->write(data);
+        if (replyStatusCode() >= 400) {
+            m_errorResponse.append(data);
+        }
         if (m_state == State::Failed) {
-            qCCritical(logCat) << getUid().toString() << "Failed to process response chunk";
+            qCCritical(logCat) << getUid().toString() << "Failed to process response chunk:" << m_sink->failReason();
         }
         // qDebug() << "Request" << m_url.toString() << "gained" << data.size() << "bytes";
     } else {
-        qCCritical(logCat) << getUid().toString() << "Cannot write download data! illegal status " << m_status;
+        qCCritical(logCat) << getUid().toString() << "Cannot write download data! illegal status" << m_status;
     }
 }
 
@@ -323,11 +373,7 @@ auto NetRequest::abort() -> bool
 {
     m_state = State::AbortedByUser;
     if (m_reply) {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 15, 0)  // QNetworkReply::errorOccurred added in 5.15
         disconnect(m_reply.get(), &QNetworkReply::errorOccurred, nullptr, nullptr);
-#else
-        disconnect(m_reply.get(), QOverload<QNetworkReply::NetworkError>::of(&QNetworkReply::error), nullptr, nullptr);
-#endif
         m_reply->abort();
     }
     return true;
@@ -352,4 +398,14 @@ QString NetRequest::errorString() const
 {
     return m_reply ? m_reply->errorString() : "";
 }
+
+void NetRequest::enableAutoRetry(bool enable)
+{
+    if (enable) {
+        m_options |= Option::AutoRetry;
+    } else {
+        m_options &= ~static_cast<int>(Option::AutoRetry);
+    }
+}
+
 }  // namespace Net
